import { __awaiter } from 'tslib';
import { animate, group, query, stagger, style, transition, trigger } from '@angular/animations';
import { isPlatformBrowser, CommonModule } from '@angular/common';
import { BehaviorSubject, of } from 'rxjs';
import { map, share } from 'rxjs/operators';
import { Inject, Injectable, Optional, PLATFORM_ID, ChangeDetectionStrategy, ChangeDetectorRef, Component, HostBinding, Input, NgZone, ViewChild, ViewEncapsulation, HostListener, Pipe, LOCALE_ID, NgModule, SkipSelf, Renderer2, Host, ApplicationRef, ComponentFactoryResolver, Injector, defineInjectable, inject, INJECTOR } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const hasV8BreakIterator = (typeof Intl !== 'undefined' && ((/** @type {?} */ (Intl))).v8BreakIterator);
class BbPlatform {
    /**
     * @param {?=} _platformId
     */
    constructor(_platformId) {
        this._platformId = _platformId;
    }
    // Browser check.
    /**
     * @return {?}
     */
    get isBrowser() {
        return this._platformId
            ? isPlatformBrowser(this._platformId)
            : typeof document === 'object' && !!document;
    }
    // Rendering engines.
    /**
     * @return {?}
     */
    get isTrident() {
        return this.isBrowser && /(msie|trident)/i.test(navigator.userAgent);
    }
    /**
     * @return {?}
     */
    get isBlink() {
        return this.isBrowser && (!!(((/** @type {?} */ (window))).chrome || hasV8BreakIterator) &&
            typeof CSS !== 'undefined' && !this.isEdge && !this.isTrident);
    }
    /**
     * @return {?}
     */
    get isWebkit() {
        return this.isBrowser && /AppleWebKit/i.test(navigator.userAgent)
            && !this.isBlink && !this.isEdge && !this.isTrident;
    }
    // Browsers.
    /**
     * @return {?}
     */
    get isChrome() {
        return this.isBrowser && /Google Inc./.test(navigator.vendor);
    }
    /**
     * @return {?}
     */
    get isEdge() {
        return this.isBrowser && /(edge)/i.test(navigator.userAgent);
    }
    /**
     * @return {?}
     */
    get isFirefox() {
        return this.isBrowser && /(firefox|minefield)/i.test(navigator.userAgent);
    }
    /**
     * @return {?}
     */
    get isSafari() {
        return this.isBrowser && /safari/i.test(navigator.userAgent) && this.isWebkit;
    }
    // Platforms.
    /**
     * @return {?}
     */
    get isIos() {
        return this.isBrowser && /iPad|iPhone|iPod/.test(navigator.userAgent) && !('MSStream' in window);
    }
    /**
     * @return {?}
     */
    get isAndroid() {
        return this.isBrowser && /android/i.test(navigator.userAgent) && !this.isTrident;
    }
}
BbPlatform.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
BbPlatform.ctorParameters = () => [
    { type: Object, decorators: [{ type: Optional }, { type: Inject, args: [PLATFORM_ID,] }] }
];
/** @nocollapse */ BbPlatform.ngInjectableDef = defineInjectable({ factory: function BbPlatform_Factory() { return new BbPlatform(inject(PLATFORM_ID, 8)); }, token: BbPlatform, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbLanguage {
    constructor() {
        this.getLanguages = (/**
         * @return {?}
         */
        () => {
            if (typeof navigator !== 'object') {
                return null;
            }
            /** @type {?} */
            let t = 'anguage';
            /** @type {?} */
            let n = navigator;
            /** @type {?} */
            let f = n['l' + t + 's'];
            /** @type {?} */
            const data = f && f.length ? f : (t = n['l' + t] ||
                n['browserL' + t] ||
                n['userL' + t]) ? [t] : t;
            return typeof data === 'string'
                ? data
                : data.join(';');
        });
    }
    /**
     * @return {?}
     */
    get all() {
        // Get the languages.
        /** @type {?} */
        const data = this.getLanguages();
        // Parse all languages by using a regex.
        /** @type {?} */
        const strings = (data || '')
            .match(/((([a-zA-Z]+(-[a-zA-Z0-9]+){0,2})|\*)(;q=[0-1](\.[0-9]+)?)?)*/g);
        // Get the content by parsing the strings.
        /** @type {?} */
        const content = strings.map((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            if (!item) {
                return null;
            }
            /** @type {?} */
            const bits = item.split(';');
            /** @type {?} */
            const ietf = bits[0].split('-');
            /** @type {?} */
            const hasScript = ietf.length === 3;
            return {
                code: ietf[0],
                script: hasScript ? ietf[1] : null,
                region: hasScript ? ietf[2] : ietf[1],
                quality: bits[1] ? parseFloat(bits[1].split('=')[1]) : 1.0
            };
        }));
        // Filter out all the empty items
        // and sort them by the quality.
        return content
            .filter((/**
         * @param {?} item
         * @return {?}
         */
        item => !!item))
            .sort((/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        (a, b) => b.quality - a.quality));
    }
}
BbLanguage.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */ BbLanguage.ngInjectableDef = defineInjectable({ factory: function BbLanguage_Factory() { return new BbLanguage(); }, token: BbLanguage, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbStorage {
    constructor() {
        this._store = {};
        // The availability of the local storage.
        this._available = false;
        this.isStorageAvailable = (/**
         * @return {?}
         */
        () => {
            if (typeof localStorage !== 'undefined') {
                try {
                    localStorage.setItem('feature_test', 'yes');
                    if (localStorage.getItem('feature_test') === 'yes') {
                        localStorage.removeItem('feature_test');
                        return true;
                    }
                    else {
                        return false;
                    }
                }
                catch (_a) {
                    return false;
                }
            }
            else {
                return false;
            }
        });
        // Validate if the localstorage is available.
        this._available = this.isStorageAvailable();
    }
    /**
     * @template T
     * @param {?} token
     * @return {?}
     */
    get(token) {
        if (!this._available) {
            /** @type {?} */
            const value = this._store[token];
            return value === undefined
                ? null
                : value;
        }
        // Retrieve the raw data from the local storage.
        /** @type {?} */
        const rawData = localStorage.getItem(token);
        // Try to parse the data.
        /** @type {?} */
        let parsedData = null;
        // Only parse the data when it is not null.
        if (rawData !== null) {
            try {
                parsedData = JSON.parse(rawData);
            }
            catch (error) {
                throw new Error('The raw data cannot be parsed; Invalid data in the storage.');
            }
        }
        // Return the parsed data.
        return parsedData;
    }
    /**
     * @template T
     * @param {?} token
     * @param {?} data
     * @return {?}
     */
    set(token, data) {
        if (!this._available) {
            this._store[token] = data;
            return;
        }
        // Stringify the data.
        /** @type {?} */
        const rawData = JSON.stringify(data);
        // Set the raw data in the storage.
        return localStorage.setItem(token, rawData);
    }
    /**
     * @param {?} token
     * @return {?}
     */
    remove(token) {
        if (!this._available) {
            delete this._store[token];
            return;
        }
        return localStorage.removeItem(token);
    }
    /**
     * @return {?}
     */
    clear() {
        // If available clear the local storage.
        if (this._available) {
            return localStorage.clear();
        }
        // Else reset the store object.
        this._store = {};
    }
}
BbStorage.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
BbStorage.ctorParameters = () => [];
/** @nocollapse */ BbStorage.ngInjectableDef = defineInjectable({ factory: function BbStorage_Factory() { return new BbStorage(); }, token: BbStorage, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbImage {
    /**
     * @param {?} _platform
     */
    constructor(_platform) {
        this._platform = _platform;
        // Data.
        this._types = ['image/png', 'image/jpeg'];
        this.calculateOffsetAndDimensions = (/**
         * @param {?} image
         * @param {?} canvasWidth
         * @param {?} canvasHeight
         * @return {?}
         */
        (image, canvasWidth, canvasHeight) => {
            /** @type {?} */
            const originalWidth = image.width;
            /** @type {?} */
            const originalHeight = image.height;
            /** @type {?} */
            const widthRatio = canvasWidth / originalWidth;
            /** @type {?} */
            const heightRatio = canvasHeight / originalHeight;
            /** @type {?} */
            let width = 0;
            /** @type {?} */
            let height = 0;
            if (originalWidth > originalHeight) {
                height = originalHeight * heightRatio;
                width = (originalWidth / originalHeight) * height;
            }
            else {
                width = originalWidth * widthRatio;
                height = (originalHeight / originalWidth) * width;
            }
            /** @type {?} */
            let xOffset = 0;
            /** @type {?} */
            let yOffset = 0;
            if (canvasHeight === height) {
                xOffset = -((width - canvasWidth) / 2);
            }
            else if (canvasWidth === width) {
                yOffset = -((height - canvasHeight) / 2);
            }
            return { xOffset, yOffset, width, height };
        });
    }
    /**
     * @param {?} contents
     * @param {?=} options
     * @return {?}
     */
    toDataUri(contents, options = null) {
        return __awaiter(this, void 0, void 0, function* () {
            // Check if the current platform is a browser.
            if (!this._platform.isBrowser) {
                return null;
            }
            // Get the image.
            /** @type {?} */
            const image = yield this.getImage(contents);
            // Get the canvas width and height.
            /** @type {?} */
            const canvasWidth = (options && options.width) || image.width;
            /** @type {?} */
            const canvasHeight = (options && options.height) || image.height;
            // Get the canvas and context.
            /** @type {?} */
            const canvas = document.createElement('canvas');
            /** @type {?} */
            const context = canvas.getContext('2d');
            canvas.width = canvasWidth;
            canvas.height = canvasHeight;
            // Check if dimensions where supplied, else create the image.
            if (!options) {
                // Draw the image.
                context.drawImage(image, 0, 0);
                // Convert the canvas to a data url.
                return canvas.toDataURL(this._types[0]);
            }
            // Get the dimensions and offsets.
            const { width, height, xOffset, yOffset } = this.calculateOffsetAndDimensions(image, canvasWidth, canvasHeight);
            // Draw the image.
            context.drawImage(image, 0, 0, image.width, image.height, xOffset, yOffset, width, height);
            // Convert the canvas to a data url.
            return canvas.toDataURL(this._types[0]);
        });
    }
    /**
     * @private
     * @param {?} contents
     * @return {?}
     */
    getImage(contents) {
        return __awaiter(this, void 0, void 0, function* () {
            // If the contents is already an HTML image element just return it.
            if (contents instanceof HTMLImageElement) {
                return contents;
            }
            // If the contents is a file convert it to an HTML image element.
            if (contents instanceof File) {
                return this.fileToImage(contents);
            }
            // If the contents is a string convert it to an HTML image element.
            if (typeof contents === 'string') {
                return this.stringToImage(contents);
            }
        });
    }
    /**
     * @private
     * @param {?} file
     * @return {?}
     */
    fileToImage(file) {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            // Check if the file matches an image.
            if (!this._types.includes(file.type)) {
                return reject('The selected file does not meet the required format.');
            }
            /** @type {?} */
            const fileReader = new FileReader();
            fileReader.readAsDataURL(file);
            fileReader.onloadend = (/**
             * @param {?} data
             * @return {?}
             */
            (data) => __awaiter(this, void 0, void 0, function* () {
                // Get the data URI.
                /** @type {?} */
                const dataUri = data.target['result'];
                // Convert the data URI to a image element.
                /** @type {?} */
                const image = yield this.stringToImage(dataUri);
                // Resolve the image.
                return resolve(image);
            }));
            fileReader.onerror = (/**
             * @param {?} error
             * @return {?}
             */
            error => reject(error));
        }));
    }
    /**
     * @private
     * @param {?} src
     * @return {?}
     */
    stringToImage(src) {
        return new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            /** @type {?} */
            const image = new Image();
            image.src = src;
            image.onload = (/**
             * @return {?}
             */
            () => resolve(image));
            image.onerror = (/**
             * @param {?} error
             * @return {?}
             */
            error => reject(error));
        }));
    }
}
BbImage.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
BbImage.ctorParameters = () => [
    { type: BbPlatform }
];
/** @nocollapse */ BbImage.ngInjectableDef = defineInjectable({ factory: function BbImage_Factory() { return new BbImage(inject(BbPlatform)); }, token: BbImage, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbInterpolator {
    constructor() {
        // Regex.
        this._checkSubstitution = /{([^}]+)}/g;
        this._replaceSubstitution = /[{}]/g;
        // Collection.
        this._middleware = {};
        this.log = (/**
         * @param {?} message
         * @return {?}
         */
        (message) => {
            console && console.warn && console.warn(message);
        });
    }
    /**
     * @param {?} value
     * @param {?} data
     * @return {?}
     */
    interpolate(value, data) {
        // Match all string inside of curly braces and
        // then filter out all the duplicate items, then
        // we should interpolate all those matches and return
        // the string to the user.
        return (value.match(this._checkSubstitution) || [])
            .filter((/**
         * @param {?} item
         * @param {?} index
         * @param {?} array
         * @return {?}
         */
        (item, index, array) => array.indexOf(item) === index))
            .reduce((/**
         * @param {?} string
         * @param {?} token
         * @return {?}
         */
        (string, token) => {
            /** @type {?} */
            const interpolated = this.match(token, data);
            if (interpolated === null) {
                return string;
            }
            return string.replace(new RegExp(token, 'g'), interpolated);
        }), value);
    }
    /**
     * @param {?} token
     * @param {?} middleware
     * @return {?}
     */
    use(token, middleware) {
        // This add the middleware function to the collection
        // so it can be used at a later time when the user
        // wants to interpolate a value with the middleware.
        this._middleware[token] = middleware;
    }
    /**
     * @private
     * @param {?} match
     * @param {?} dictionary
     * @return {?}
     */
    match(match, dictionary) {
        // Remove the brackets from the match
        // so we get the data between those brackets.
        /** @type {?} */
        const data = match.replace(this._replaceSubstitution, '');
        // Get the token and the methods by splitting
        // on the pipe symbol.
        const [token, ...methods] = data.split('|').map((/**
         * @param {?} item
         * @return {?}
         */
        item => item.trim()));
        // Get the value for the token from
        // the dictionary.
        /** @type {?} */
        const value = dictionary[token];
        // Check if the value is undefined, if so
        // we should return null and not proceed.
        if (typeof value === 'undefined') {
            this.log(`Interpolator: value was undefined for token: "${token}".`);
            return null;
        }
        // Loop through all the middleware methods and try to
        // conform the string to the given parameters.
        return methods.reduce((/**
         * @param {?} string
         * @param {?} middlewareName
         * @return {?}
         */
        (string, middlewareName) => {
            // Try to retrieve the middleware from the collection.
            /** @type {?} */
            const middleware = this._middleware[middlewareName];
            // If the middleware is missing return an error
            // so that the user knows it was not found.
            if (!middleware) {
                this.log(`Interpolator: cannot find middleware with name: "${middlewareName}".`);
                return string;
            }
            // Execute the middleware function with the string.
            return middleware(string);
        }), `${value}`);
    }
}
BbInterpolator.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */ BbInterpolator.ngInjectableDef = defineInjectable({ factory: function BbInterpolator_Factory() { return new BbInterpolator(); }, token: BbInterpolator, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbDashboard {
    /**
     * @param {?} _renderer
     */
    constructor(_renderer) {
        this._renderer = _renderer;
        // Inputs.
        this.backgroundColor = '#d5dae6';
        this.maxWidth = '1200px';
        this.padding = '20px';
        // State.
        this.sidebarVisible$ = new BehaviorSubject(true);
    }
    /**
     * @return {?}
     */
    get visibility() {
        return this.sidebarVisible$.asObservable();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // Set the HTML class to prevent scrolling.
        return this.setHtmlClass();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        // Remove the HTML class.
        return this.setHtmlClass(true);
    }
    /**
     * @return {?}
     */
    toggleSidebarVisibility() {
        // Get the previous value.
        /** @type {?} */
        const previousValue = this.sidebarVisible$.getValue();
        // Invert the value and set the visibility.
        this.setSidebarVisibility(!previousValue);
    }
    /**
     * @param {?} newValue
     * @return {?}
     */
    setSidebarVisibility(newValue) {
        this.sidebarVisible$.next(newValue);
    }
    /**
     * @private
     * @param {?=} shouldRemove
     * @return {?}
     */
    setHtmlClass(shouldRemove = false) {
        // Check if the document element is available.
        if (!document || !document.documentElement) {
            return;
        }
        // Define the class name.
        /** @type {?} */
        const className = 'bb-dashboard-html';
        // Add or remove the class.
        return shouldRemove
            ? this._renderer.removeClass(document.documentElement, className)
            : this._renderer.addClass(document.documentElement, className);
    }
}
BbDashboard.decorators = [
    { type: Component, args: [{
                selector: 'bb-dashboard',
                template: "<!--\n    The outlet for where the sidebar\n    will be placed (if provided).\n-->\n<ng-content select=\"bb-dashboard-sidebar\">\n</ng-content>\n\n<!--\n    The section containing the header\n    and the main content.\n-->\n<section class=\"bb-dashboard-section\">\n    <!--\n        The outlet for where the header\n        will be placed (if provided).\n    -->\n    <ng-content select=\"bb-dashboard-header\">\n    </ng-content>\n\n    <!--\n        The outlet of the content that\n        will be provided by the router.\n    -->\n    <main [style.background-color]=\"backgroundColor\"\n          [style.padding]=\"padding\"\n          class=\"bb-dashboard-scroll\">\n        <div [style.max-width]=\"maxWidth\"\n             class=\"bb-dashboard-container\">\n            <ng-content></ng-content>\n        </div>\n    </main>\n</section>\n\n<!--\n    The backdrop that will be visible when\n    the sidebar is opened (only on mobile).\n-->\n<div [class.bb-dashboard-backdrop--visible]=\"!(visibility | async)\"\n     (click)=\"toggleSidebarVisibility()\"\n     class=\"bb-dashboard-backdrop\">\n</div>",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dashboard'
                },
                styles: [".bb-dashboard{height:100vh;display:-ms-flexbox;display:flex;overflow:hidden;position:relative}.bb-dashboard-section{-ms-flex:1;flex:1;height:100vh;display:-ms-flexbox;display:flex;overflow-x:hidden;-ms-flex-direction:column;flex-direction:column}.bb-dashboard-scroll{-ms-flex:1;flex:1;overflow-x:hidden;overflow-y:scroll;-webkit-overflow-scrolling:touch}.bb-dashboard-container{width:100%;margin:0 auto}.bb-dashboard-backdrop{top:0;left:0;right:0;bottom:0;opacity:0;z-index:5;position:absolute;will-change:opacity;pointer-events:none;background-color:rgba(0,0,0,.5);transition:opacity .2s cubic-bezier(0,0,.2,1)}.bb-dashboard-backdrop--visible{opacity:1;cursor:pointer;pointer-events:all}@media only screen and (min-width:768px){.bb-dashboard-backdrop{display:none}}.bb-dashboard-html{height:100%;overflow:hidden}"]
            }] }
];
/** @nocollapse */
BbDashboard.ctorParameters = () => [
    { type: Renderer2 }
];
BbDashboard.propDecorators = {
    backgroundColor: [{ type: Input }],
    maxWidth: [{ type: Input }],
    padding: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbDashboardHeader {
    /**
     * @param {?} _parent
     */
    constructor(_parent) {
        this._parent = _parent;
    }
    /**
     * @return {?}
     */
    get isSidebarVisible() {
        return this._parent
            ? this._parent.visibility
            : of(false);
    }
    /**
     * @return {?}
     */
    onToggleSidebarClicked() {
        // If the parent exists toggle the sidebar visibility.
        if (!this._parent) {
            return;
        }
        this._parent.toggleSidebarVisibility();
    }
}
BbDashboardHeader.decorators = [
    { type: Component, args: [{
                selector: 'bb-dashboard-header',
                template: "<!--\n    The button that toggles the sidebar.\n-->\n<button (click)=\"onToggleSidebarClicked()\"\n        [class.bb-dashboard-header-button--reversed]=\"isSidebarVisible | async\"\n        class=\"bb-dashboard-header-button\"\n        type=\"button\"\n        tabindex=\"0\"\n        role=\"switch\">\n    <svg xmlns=\"http://www.w3.org/2000/svg\"\n         viewBox=\"0 0 157 109\"\n         width=\"17\"\n         height=\"12\"\n         role=\"img\">\n        <title>\n            Toggle\n        </title>\n        <path fill=\"#fff\"\n              d=\"M82 42V11.2L30 54.5l52 43.3V67h75V42zM9.5 109h-2c-4.1 0-7.5-3.4-7.5-7.5v-94C0 3.4 3.4 0 7.5 0h2C13.6 0 17 3.4 17 7.5v94c0 4.1-3.4 7.5-7.5 7.5z\">\n        </path>\n    </svg>\n</button>\n\n<!--\n    The extra items of the header.\n-->\n<div class=\"bb-dashboard-header-items\">\n    <ng-content></ng-content>\n</div>",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dashboard-header'
                },
                styles: [".bb-dashboard-header{z-index:1;width:100%;color:#fff;height:50px;display:-ms-flexbox;display:flex;padding:0 8px;min-height:50px;-ms-flex-align:center;align-items:center;background-color:#3f51b5;border-bottom:1px solid #303f9f}.bb-dashboard-header-button{width:68px;height:34px;border-radius:4px;-ms-flex-align:center;align-items:center;display:-ms-inline-flexbox;display:inline-flex;-ms-flex-pack:center;justify-content:center;border:1px solid #283593;background-color:transparent}.bb-dashboard-header-button>svg{height:100%;-webkit-transform:none;-ms-transform:none;transform:none;transition:transform .2s cubic-bezier(0,0,.2,1);transition:transform .2s cubic-bezier(0,0,.2,1),-webkit-transform .2s cubic-bezier(0,0,.2,1)}.bb-dashboard-header-button--reversed>svg{-webkit-transform:rotateY(180deg);transform:rotateY(180deg)}.bb-dashboard-header-button:hover{background-color:rgba(0,0,0,.04)}.bb-dashboard-header-button:active{background-color:rgba(0,0,0,.08)}.bb-dashboard-header-items{display:-ms-flexbox;display:flex;margin-left:auto;-ms-flex-align:center;align-items:center}.bb-dashboard-header-items>:not(:first-child){margin-left:8px}@media only screen and (min-width:768px){.bb-dashboard-header-button>svg{-webkit-transform:rotateY(180deg);transform:rotateY(180deg)}.bb-dashboard-header-button--reversed>svg{-webkit-transform:none;-ms-transform:none;transform:none}}"]
            }] }
];
/** @nocollapse */
BbDashboardHeader.ctorParameters = () => [
    { type: BbDashboard, decorators: [{ type: Optional }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbDashboardSidebar {
    /**
     * @param {?} _platform
     * @param {?} _parent
     */
    constructor(_platform, _parent) {
        this._platform = _platform;
        this._parent = _parent;
        // Bindings.
        this.visible = true;
        // State.
        this._state$ = new BehaviorSubject([]);
        // Token.
        this._localStorageToken = 'bb-dashboard-sidebar-state';
        this.saveCache = (/**
         * @param {?} data
         * @return {?}
         */
        (data) => {
            // Get the local storage.
            /** @type {?} */
            const localStorage = this.getLocalStorage();
            // Check if the local storage exists.
            if (!localStorage) {
                return;
            }
            // Set the cache in the local storage.
            try {
                localStorage.setItem(this._localStorageToken, JSON.stringify(data));
            }
            catch (error) {
                console.log('Error setting local storage data.');
            }
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // Retrieve and set the cache.
        this.setCache();
        // Get the visibility observable from the parent
        // if the parent exists.
        /** @type {?} */
        const observable$ = this._parent
            ? this._parent.visibility
            : of(false);
        // Subscribe to get the state changes.
        this._subscription = observable$
            .subscribe((/**
         * @param {?} value
         * @return {?}
         */
        value => this.visible = value));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        // Unsubscribe to prevent memory leaks.
        this._subscription && this._subscription.unsubscribe();
    }
    /**
     * @param {?} id
     * @return {?}
     */
    registerGroup(id) {
        // Get the current groups.
        /** @type {?} */
        const current = this._state$.getValue();
        // Check if the id is already inside the groups.
        if (current.findIndex((/**
         * @param {?} item
         * @return {?}
         */
        item => item.id === id)) > -1) {
            throw new Error('Group id was already defined; can only have distinct id\'s.');
        }
        // Check if the item is in the cache
        // and create a new entry.
        /** @type {?} */
        const cacheItem = this._cache.find((/**
         * @param {?} item
         * @return {?}
         */
        item => item.id === id));
        /** @type {?} */
        const newEntry = {
            id: id,
            state: cacheItem ? cacheItem.state : 'opened'
        };
        // Push the new groups array.
        this._state$.next([...current, newEntry]);
        // Return the slice of the state that the group needs.
        return this._state$.pipe(map((/**
         * @param {?} state
         * @return {?}
         */
        state => state.find((/**
         * @param {?} item
         * @return {?}
         */
        item => item.id === id)))), map((/**
         * @param {?} group
         * @return {?}
         */
        group$$1 => group$$1.state)));
    }
    /**
     * @param {?} id
     * @return {?}
     */
    unregisterGroup(id) {
        // Get the old groups.
        /** @type {?} */
        const oldValue = this._state$.getValue();
        // Calculate the new groups.
        /** @type {?} */
        const newValue = oldValue.filter((/**
         * @param {?} item
         * @return {?}
         */
        item => item.id !== id));
        // Push the new groups array.
        this._state$.next(newValue);
    }
    /**
     * @param {?} id
     * @param {?} newState
     * @return {?}
     */
    toggleGroup(id, newState) {
        // Get the old groups.
        /** @type {?} */
        const oldValue = this._state$.getValue();
        // Create the new groups.
        /** @type {?} */
        const newValue = oldValue.map((/**
         * @param {?} item
         * @return {?}
         */
        item => {
            return item.id !== id
                ? item
                : { id: id, state: newState };
        }));
        // Push the new groups array.
        this._state$.next(newValue);
        // Save in the cache.
        this.saveCache(newValue);
    }
    /**
     * @return {?}
     */
    close() {
        // Set the sidebar visibility to true, this means
        // that on mobile the sidebar will be closed.
        this._parent.setSidebarVisibility(true);
    }
    /**
     * @private
     * @return {?}
     */
    getLocalStorage() {
        // Check if the current platform is a browser.
        if (!this._platform.isBrowser) {
            return null;
        }
        // Check if the window and local storage exist.
        return window && window.localStorage
            ? localStorage
            : null;
    }
    /**
     * @private
     * @return {?}
     */
    setCache() {
        // Get the local storage.
        /** @type {?} */
        const localStorage = this.getLocalStorage();
        // Check if the local storage exists.
        if (!localStorage) {
            return;
        }
        // Retrieve the raw data from the storage.
        /** @type {?} */
        const rawData = localStorage.getItem(this._localStorageToken);
        // Try to parse the data.
        /** @type {?} */
        let parsedData = null;
        // Only parse the data when it is not null.
        if (rawData !== null) {
            try {
                parsedData = JSON.parse(rawData);
            }
            catch (error) {
                console.log('Error parsing local storage data.');
            }
        }
        // Set the cache to the parsed data.
        this._cache = parsedData || [];
    }
}
BbDashboardSidebar.decorators = [
    { type: Component, args: [{
                selector: 'bb-dashboard-sidebar',
                template: "<!--\n    The header of the sidebar.\n-->\n<header class=\"bb-dashboard-sidebar-header\">\n    <ng-content>\n    </ng-content>\n</header>\n\n<!--\n    The scroll area that contains the\n    groups in the navigation.\n-->\n<div class=\"bb-dashboard-sidebar-scroll\">\n    <ng-content select=\"bb-dashboard-sidebar-group\"></ng-content>\n</div>",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dashboard-sidebar',
                    'role': 'navigation'
                },
                styles: [".bb-dashboard-sidebar{top:0;bottom:0;z-index:10;left:-256px;width:256px;height:100vh;display:block;position:fixed;min-width:256px;will-change:transform;-webkit-transform:translateX(256px);-ms-transform:translateX(256px);transform:translateX(256px);border-right:1px solid rgba(0,0,0,.4);transition:transform 250ms cubic-bezier(0,0,.2,1);transition:transform 250ms cubic-bezier(0,0,.2,1),-webkit-transform 250ms cubic-bezier(0,0,.2,1);background:linear-gradient(to bottom,#fff,#f2f2f2) #fff}.bb-dashboard-sidebar.visible{-webkit-transform:none;-ms-transform:none;transform:none}.bb-dashboard-sidebar-header{height:120px;display:-ms-flexbox;display:flex;-ms-flex-align:center;align-items:center;-ms-flex-pack:center;justify-content:center;border-bottom:1px solid #d8d8d8;background-color:rgba(255,255,255,.5)}.bb-dashboard-sidebar-scroll{overflow:auto;height:calc(100vh - 120px)}@media only screen and (min-width:768px){.bb-dashboard-sidebar{left:0;z-index:auto;-webkit-transform:none;-ms-transform:none;transform:none;position:relative;margin-left:-256px;transition:margin-left 250ms cubic-bezier(0,0,.2,1)}.bb-dashboard-sidebar.visible{margin-left:0}}"]
            }] }
];
/** @nocollapse */
BbDashboardSidebar.ctorParameters = () => [
    { type: BbPlatform },
    { type: BbDashboard, decorators: [{ type: Optional }, { type: Host }] }
];
BbDashboardSidebar.propDecorators = {
    visible: [{ type: HostBinding, args: ['class.visible',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbDashboardSidebarGroup {
    /**
     * @param {?} _parent
     */
    constructor(_parent) {
        this._parent = _parent;
        // Inputs.
        this.id = null;
        this.icon = null;
        this.title = null;
        // State.
        this.state = 'opened';
    }
    // Host bindings.
    /**
     * @return {?}
     */
    get isExpanded() {
        return this.state === 'opened';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // Check if the id and title have been supplied.
        if (!this.id || !this.title) {
            throw new Error('Id and title are required.');
        }
        // Register the group at the parent and
        // retrieve the slice of the state.
        /** @type {?} */
        const observable$ = this._parent
            ? this._parent.registerGroup(this.id)
            : of('opened');
        // Subscribe to the state.
        this._subscription = observable$
            .subscribe((/**
         * @param {?} state
         * @return {?}
         */
        state => this.state = state));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        // Unsubscribe to prevent memory leaks.
        this._subscription && this._subscription.unsubscribe();
        // Unregister the group at the parent.
        this._parent && this._parent.unregisterGroup(this.id);
    }
    /**
     * @return {?}
     */
    onClick() {
        // Change the state.
        /** @type {?} */
        const nextState = this.state === 'opened' ? 'closed' : 'opened';
        // Set the state of the sidebar.
        this._parent && this._parent.toggleGroup(this.id, nextState);
    }
    /**
     * @return {?}
     */
    onItemClicked() {
        // Close the sidebar on the parent.
        this._parent && this._parent.close();
    }
}
BbDashboardSidebarGroup.decorators = [
    { type: Component, args: [{
                selector: 'bb-dashboard-sidebar-group',
                template: "<!--\n    The wrapper that contains the title and icon.\n-->\n<button (click)=\"onClick()\"\n        class=\"bb-dashboard-sidebar-group-title\"\n        type=\"button\"\n        role=\"switch\">\n    <i *ngIf=\"icon\"\n       class=\"material-icons bb-dashboard-sidebar-group-icon\">\n        {{ icon }}\n    </i>\n    <span *ngIf=\"title\"\n          class=\"bb-dashboard-sidebar-group-text\">\n        {{ title }}\n    </span>\n    <span class=\"bb-dashboard-sidebar-group-toggle\">\n        <svg xmlns=\"http://www.w3.org/2000/svg\"\n             viewBox=\"0 0 157 109\"\n             width=\"16\"\n             height=\"16\"\n             role=\"img\">\n            <title>\n                Toggle\n            </title>\n            <path\n                d=\"M82 42V11.2L30 54.5l52 43.3V67h75V42zM9.5 109h-2c-4.1 0-7.5-3.4-7.5-7.5v-94C0 3.4 3.4 0 7.5 0h2C13.6 0 17 3.4 17 7.5v94c0 4.1-3.4 7.5-7.5 7.5z\">\n            </path>\n        </svg>\n    </span>\n</button>\n\n<!--\n    The sidebar items that contain the\n    links to navigate to other pages.\n-->\n<ng-content select=\"[bb-dashboard-sidebar-item]\">\n</ng-content>",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dashboard-sidebar-group',
                    'role': 'group'
                },
                styles: [".bb-dashboard-sidebar-group{color:#000;display:block;overflow:hidden;border-bottom:1px solid rgba(0,0,0,.1);transition:background-color .2s cubic-bezier(0,0,.2,1),padding-bottom .2s cubic-bezier(0,0,.2,1)}.bb-dashboard-sidebar-group.expanded{padding-bottom:20px;background-color:rgba(0,0,0,.04)}.bb-dashboard-sidebar-group.expanded .bb-dashboard-sidebar-group-title{color:#000}.bb-dashboard-sidebar-group.expanded .bb-dashboard-sidebar-item{opacity:1;height:32px;line-height:32px;padding:6px 16px 6px 54px}.bb-dashboard-sidebar-group.expanded .bb-dashboard-sidebar-group-toggle>svg{-webkit-transform:rotate(90deg);-ms-transform:rotate(90deg);transform:rotate(90deg)}.bb-dashboard-sidebar-group-title{margin:0;width:100%;border:none;display:-ms-flexbox;display:flex;cursor:pointer;text-align:left;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;-ms-flex-align:center;align-items:center;padding:17px 15px;color:rgba(0,0,0,.8);background-color:transparent;transition:color .2s cubic-bezier(0,0,.2,1)}.bb-dashboard-sidebar-group-title:focus>.bb-dashboard-sidebar-group-toggle,.bb-dashboard-sidebar-group-title:hover>.bb-dashboard-sidebar-group-toggle{opacity:1}.bb-dashboard-sidebar-group-title:active>.bb-dashboard-sidebar-group-toggle{background-color:rgba(0,0,0,.08)}.bb-dashboard-sidebar-group-icon{margin-right:15px;color:rgba(0,0,0,.25)}.bb-dashboard-sidebar-group-text{width:100%;display:block;font-size:14px;max-width:100%;overflow:hidden;font-weight:500;line-height:15px;white-space:nowrap;text-overflow:ellipsis;text-transform:uppercase}.bb-dashboard-sidebar-group-toggle{opacity:0;width:30px;height:30px;display:-ms-flexbox;display:flex;min-width:30px;min-height:30px;border-radius:50%;-ms-flex-align:center;align-items:center;-ms-flex-pack:center;justify-content:center;transition:opacity .2s cubic-bezier(0,0,.2,1),background-color .2s cubic-bezier(0,0,.2,1)}.bb-dashboard-sidebar-group-toggle>svg{width:16px;height:16px;fill:rgba(0,0,0,.25);-webkit-transform:rotate(90deg) rotateY(180deg);transform:rotate(90deg) rotateY(180deg);transition:transform .2s cubic-bezier(0,0,.2,1);transition:transform .2s cubic-bezier(0,0,.2,1),-webkit-transform .2s cubic-bezier(0,0,.2,1)}"]
            }] }
];
/** @nocollapse */
BbDashboardSidebarGroup.ctorParameters = () => [
    { type: BbDashboardSidebar, decorators: [{ type: Optional }, { type: Host }] }
];
BbDashboardSidebarGroup.propDecorators = {
    isExpanded: [{ type: HostBinding, args: ['class.expanded',] }],
    id: [{ type: Input }],
    icon: [{ type: Input }],
    title: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbDashboardSidebarItem {
    /**
     * @param {?} _parent
     */
    constructor(_parent) {
        this._parent = _parent;
        // Inputs.
        this.tabIndex = 0;
    }
    /**
     * @return {?}
     */
    onClick() {
        // Trigger the parent on item clicked call
        // so that the sidebar can be closed.
        this._parent && this._parent.onItemClicked();
    }
    /**
     * @return {?}
     */
    get attributeTabIndex() {
        // Make the items tabindex -1 so it can't
        // be tabbed to when the parent is not expanded.
        if (!this._parent.isExpanded) {
            return -1;
        }
        // Else return the specified tabindex.
        return this.tabIndex;
    }
}
BbDashboardSidebarItem.decorators = [
    { type: Component, args: [{
                selector: '[bb-dashboard-sidebar-item]',
                template: "<!--\n    The content between the tags will\n    be reflected here.\n-->\n<span class=\"bb-dashboard-sidebar-item-content\">\n    <ng-content>\n    </ng-content>\n</span>",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-dashboard-sidebar-item',
                    '[attr.tabindex]': 'attributeTabIndex'
                },
                styles: [".bb-dashboard-sidebar-item{height:0;opacity:0;padding:0;display:-ms-flexbox;display:flex;line-height:0;cursor:pointer;font-size:14px;font-weight:500;margin-right:20px;-ms-flex-align:center;align-items:center;text-decoration:none;will-change:opacity,height;border-top-right-radius:16px;border-bottom-right-radius:16px;color:rgba(0,0,0,.85)!important;transition:height .2s cubic-bezier(0,0,.2,1),opacity .2s cubic-bezier(0,0,.2,1) 50ms,color .2s cubic-bezier(0,0,.2,1),background-color .2s cubic-bezier(0,0,.2,1)}.bb-dashboard-sidebar-item:focus,.bb-dashboard-sidebar-item:hover{background-color:rgba(0,0,0,.05)}.bb-dashboard-sidebar-item:active{background-color:rgba(0,0,0,.1)}.bb-dashboard-sidebar-item.active,.bb-dashboard-sidebar-item.active:active,.bb-dashboard-sidebar-item.active:focus,.bb-dashboard-sidebar-item.active:hover{color:#000;background-color:rgba(0,0,0,.15)}.bb-dashboard-sidebar-item-content{overflow:hidden;max-width:100%;white-space:nowrap;text-overflow:ellipsis}"]
            }] }
];
/** @nocollapse */
BbDashboardSidebarItem.ctorParameters = () => [
    { type: BbDashboardSidebarGroup, decorators: [{ type: Optional }] }
];
BbDashboardSidebarItem.propDecorators = {
    tabIndex: [{ type: Input }],
    onClick: [{ type: HostListener, args: ['click',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const DECLARATIONS_EXPORTS = [
    BbDashboard,
    BbDashboardHeader,
    BbDashboardSidebar,
    BbDashboardSidebarGroup,
    BbDashboardSidebarItem
];
class BbDashboardModule {
}
BbDashboardModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [...DECLARATIONS_EXPORTS],
                exports: [...DECLARATIONS_EXPORTS]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const BbNotificationType = {
    Success: 'success',
    Error: 'error',
    Warning: 'warning',
    Info: 'info',
    Custom: 'custom',
};
class BbNotificationsConfig {
}
/** @enum {string} */
const BbNotificationsPosition = {
    TopRight: 'flex-start|flex-end',
    BottomRight: 'flex-end|flex-end',
    TopLeft: 'flex-start|flex-start',
    BottomLeft: 'flex-end|flex-start',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const s = '250ms cubic-bezier(0, 0, .2, 1)';
/** @type {?} */
const l = '400ms cubic-bezier(0, 0, .2, 1)';
/** @type {?} */
const notificationAnimation = trigger('notificationListAnimation', [
    transition('* => *', [
        query(':enter', group([
            // 1. Set the initial state.
            style({ height: 0, opacity: 0, transform: 'translateX({{ to }}%) scale(0.95)' }),
            // 2. Start the animation to show the item.
            stagger(0, [
                animate(s, style({ height: '*' })),
                animate(l, style({ opacity: 1, transform: 'translateX(0) scale(0.95)' })),
                animate(s, style({ transform: 'translateX(0) scale(1)' }))
            ])
        ]), { optional: true }),
        query(':leave', group([
            // 1. Set the initial state.
            style({ height: '*', opacity: 1, transform: 'translateX(0) scale(1)' }),
            // 2. Start the animation to hide the item.
            stagger(0, [
                animate(s, style({ transform: 'translateX(0) scale(0.95)' })),
                animate(l, style({ opacity: 0, transform: 'translateX({{ to }}%) scale(0.95)' })),
                animate(s, style({ height: 0 }))
            ])
        ]), { optional: true })
    ])
]);

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbNotificationsList {
    constructor() {
        // Track by function.
        this.trackByFn = (/**
         * @param {?} _
         * @param {?} item
         * @return {?}
         */
        (_, item) => item.id);
    }
    /**
     * @return {?}
     */
    get horizontalPosition() {
        return this.location.horizontal;
    }
    /**
     * @return {?}
     */
    get verticalPosition() {
        return this.location.vertical;
    }
    /**
     * @return {?}
     */
    get animation() {
        /** @type {?} */
        const to = this.horizontalPosition === 'flex-start'
            ? -50
            : 50;
        return { to };
    }
    /**
     * @private
     * @return {?}
     */
    get location() {
        const [y, x] = (this.position || '').split('|');
        return { vertical: y || null, horizontal: x || null };
    }
}
BbNotificationsList.decorators = [
    { type: Component, args: [{
                selector: 'bb-notifications-list',
                template: "<!--\n    The notifications list.\n\n    The list provides the animation effect for the\n    incoming notifications.\n-->\n<div *ngIf=\"notifications$ | async as notifications\"\n     [@notificationListAnimation]=\"{value: notifications?.length, params: animation}\"\n     class=\"notifications-list-wrapper\">\n    <!--\n        The loop for the notifications.\n\n        All notifications will be displayed here. They\n        are wrapped within a wrapper which separates them.\n    -->\n    <div *ngFor=\"let notification of notifications; trackBy: trackByFn\"\n         class=\"notifications-item-wrapper\">\n        <bb-notifications-item [notification]=\"notification\"\n                               [dismissButtonText]=\"dismissText\">\n        </bb-notifications-item>\n    </div>\n</div>\n",
                animations: [notificationAnimation],
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'class': 'bb-notifications-list',
                    'role': 'list'
                },
                styles: [".bb-notifications-list{top:0;left:0;right:0;bottom:0;z-index:500;padding:10px;display:-ms-flexbox;display:flex;position:fixed;pointer-events:none}.notifications-list-wrapper{width:100%;display:block;max-width:400px;position:relative}.notifications-item-wrapper{padding:0;margin:0 0 10px;pointer-events:all}"]
            }] }
];
BbNotificationsList.propDecorators = {
    horizontalPosition: [{ type: HostBinding, args: ['style.justify-content',] }],
    verticalPosition: [{ type: HostBinding, args: ['style.align-items',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbNotificationsItem {
    /**
     * @param {?} _zone
     * @param {?} _platform
     * @param {?} _changeDetection
     */
    constructor(_zone, _platform, _changeDetection) {
        this._zone = _zone;
        this._platform = _platform;
        this._changeDetection = _changeDetection;
        // Data.
        this._progress = 0;
        this._count = 0;
        // Data.
        this._radius = 19;
        this._circumference = this._radius * 2 * Math.PI;
        this.callActionAndDestroy = (/**
         * @param {?} notification
         * @param {?} action
         * @return {?}
         */
        (notification, action) => {
            // Perform the callback (if it exists).
            action && action.callback && action.callback();
            // Call the destroy method (if it exists).
            notification && notification.destroy && notification.destroy();
        });
        this.instance = (/**
         * @return {?}
         */
        () => {
            // Calculate the difference.
            this._difference = (new Date().getTime() - this._startTime) - (this._count * this._speed);
            // Add up the count.
            if (this._count++ === this._steps) {
                this.notification.destroy();
            }
            // Add the steps to the progress.
            this._progress += 100 / this._steps;
            // Set a new timer.
            this._timer = this.setTimeout(this.instance, this._speed - this._difference);
            // Run a new change detection cycle.
            this._zone.run((/**
             * @return {?}
             */
            () => this._changeDetection.detectChanges()));
        });
        this.setTimeout = (/**
         * @param {?} method
         * @param {?} timeout
         * @return {?}
         */
        (method, timeout) => {
            // Check if the window and method exist.
            if (!window || !window.setTimeout) {
                return null;
            }
            return window.setTimeout(method, timeout);
        });
    }
    /**
     * @return {?}
     */
    get getClass() {
        return `bb-notifications-item ${this.notification.type}`;
    }
    /**
     * @return {?}
     */
    get progressStrokeArray() {
        /** @type {?} */
        const value = Math.floor(this._circumference);
        return `${value}, ${value}`;
    }
    /**
     * @return {?}
     */
    get progressStrokeOffset() {
        return Math.floor(this._circumference - this._progress / 100 * this._circumference);
    }
    /**
     * @return {?}
     */
    get isContentString() {
        return typeof this.notification.content === 'string';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // Check if the notification timeout is not 0 and the platform is a browser.
        if (this.notification.timeout <= 0 || !this._platform.isBrowser) {
            this.notification.dismiss = true;
            return;
        }
        // Start the timeout.
        this.startTimeout();
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        // Clear the timeout if it was set.
        this._timer && window && window.clearTimeout && window.clearTimeout(this._timer);
    }
    /**
     * @private
     * @return {?}
     */
    startTimeout() {
        // Calculate the steps of the timeout.
        this._steps = this.notification.timeout / 10;
        // Calculate the speed of the timeout.
        this._speed = this.notification.timeout / this._steps;
        // Get the start time.
        this._startTime = new Date().getTime();
        // Set a new timer outside of Angular.
        this._zone.runOutsideAngular((/**
         * @return {?}
         */
        () => this._timer = this.setTimeout(this.instance, this._speed)));
    }
}
BbNotificationsItem.decorators = [
    { type: Component, args: [{
                selector: 'bb-notifications-item',
                template: "<div class=\"notification-content-wrapper\">\n    <!--\n        The icon for the notification.\n\n        The icon is reflected by the type of the\n        notification. Same goes for the color and\n        background-color.\n    -->\n    <div [style.color]=\"notification?.color\"\n         [style.background-color]=\"notification?.backgroundColor\"\n         class=\"notification-icon-wrapper\">\n        <svg xmlns=\"http://www.w3.org/2000/svg\"\n             viewBox=\"0 0 16 19.5\"\n             class=\"notification-icon\">\n            <path\n                d=\"M8 19.5c1.1 0 2-.9 2-2H6c0 1.1.9 2 2 2zM14 13.5v-5c0-3.1-1.6-5.6-4.5-6.3v-.7C9.5.7 8.8 0 8 0S6.5.7 6.5 1.5v.7C3.6 2.9 2 5.4 2 8.5v5l-2 2v1h16v-1l-2-2z\">\n            </path>\n        </svg>\n\n        <!--\n            The circular progress for the notification.\n\n            The progress is only visible when a timeout is\n            set. The progress adapts to the time remaining\n            for the notification.\n        -->\n        <svg *ngIf=\"notification?.timeout > 0\"\n             class=\"notification-progress-ring\"\n             viewBox=\"0 0 40 40\"\n             role=\"progressbar\">\n            <circle #progress\n                    [attr.stroke-dasharray]=\"progressStrokeArray\"\n                    [attr.stroke-dashoffset]=\"progressStrokeOffset\"\n                    [style.stroke]=\"notification?.color\"\n                    class=\"notification-progress-circle\"\n                    stroke-width=\"2\"\n                    fill=\"transparent\"\n                    r=\"19\"\n                    cx=\"20\"\n                    cy=\"20\">\n            </circle>\n        </svg>\n    </div>\n\n    <!--\n        The content for the notification.\n\n        This displays a string or a template based\n        on the given parameter by the user.\n    -->\n    <div class=\"notification-content\">\n        <ng-container *ngIf=\"isContentString; else templateContent\">\n            {{ notification?.content }}\n        </ng-container>\n    </div>\n</div>\n\n<!--\n    The actions of the notification.\n\n    All actions are displayed here and are\n    based on the actions array the user provided.\n-->\n<div *ngIf=\"notification?.actions?.length > 0 || notification?.dismiss\"\n     class=\"notification-actions\">\n    <button *ngFor=\"let action of notification?.actions\"\n            (click)=\"callActionAndDestroy(notification, action)\"\n            type=\"button\"\n            [class.destructive]=\"action?.type === 'cancel'\"\n            class=\"notification-actions-button\">\n        <span class=\"notification-actions-button-highlight\">{{ action?.title }}</span>\n    </button>\n    <button *ngIf=\"notification?.dismiss\"\n            (click)=\"notification?.destroy()\"\n            class=\"notification-actions-button destructive\"\n            type=\"button\">\n        <span class=\"notification-actions-button-highlight\">{{ notification?.dismissText || dismissButtonText }}</span>\n    </button>\n</div>\n\n<!--\n    The outlet for the notification content template ref.\n-->\n<ng-template #templateContent>\n    <ng-container *ngTemplateOutlet=\"notification?.content\"></ng-container>\n</ng-template>\n",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                preserveWhitespaces: false,
                host: {
                    'role': 'alert'
                },
                styles: [".bb-notifications-item{display:-ms-flexbox;display:flex;border-radius:4px;-ms-flex-direction:column;flex-direction:column;background-color:#fff;border:1px solid #d6dbe0}.bb-notifications-item.success .notification-icon-wrapper{color:#2dc05d;background-color:#ebfaf0}.bb-notifications-item.success .notification-progress-circle{stroke:#36d068}.bb-notifications-item.error .notification-icon-wrapper{color:#f43e3e;background-color:#feeeee}.bb-notifications-item.error .notification-progress-circle{stroke:#f55656}.bb-notifications-item.warning .notification-icon-wrapper{color:#f1ae00;background-color:#fff8e7}.bb-notifications-item.warning .notification-progress-circle{stroke:#ffbc0b}.bb-notifications-item.info .notification-icon-wrapper{color:#099bf6;background-color:#e9f6fe}.bb-notifications-item.info .notification-progress-circle{stroke:#22a5f7}.bb-notifications-item.custom .notification-icon-wrapper{color:#657385;background-color:#f3f5f6}.bb-notifications-item.custom .notification-progress-circle{stroke:#657385}.notification-content-wrapper{padding:20px;display:-ms-flexbox;display:flex;-ms-flex-align:center;align-items:center}.notification-icon-wrapper{width:40px;height:40px;display:-ms-flexbox;display:flex;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;position:relative;border-radius:50%;-ms-flex-align:center;align-items:center;-ms-flex-pack:center;justify-content:center;color:#657385;background-color:#f3f5f6}.notification-icon{width:100%;height:100%;padding:10px;fill:currentColor}.notification-content{-ms-flex:1;flex:1;font-size:14px;line-height:1.5;font-weight:500;margin-left:20px;color:rgba(0,0,0,.85)}.notification-content i{font-style:italic}.notification-actions-button,.notification-content{font-family:inherit}.notification-actions{display:-ms-flexbox;display:flex;margin:0 20px;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;border-top:1px solid #f3f5f6}.notification-actions-button{-ms-flex:1;flex:1;border:none;color:#22a5f7;cursor:pointer;font-size:14px;font-weight:500;padding:12px 2px;text-align:center;background-color:transparent}.notification-actions-button:focus>.notification-actions-button-highlight,.notification-actions-button:hover>.notification-actions-button-highlight{background-color:rgba(34,165,247,.1)}.notification-actions-button:active>.notification-actions-button-highlight{background-color:rgba(34,165,247,.2)}.notification-actions-button.destructive{color:#f55656}.notification-actions-button.destructive:focus>.notification-actions-button-highlight,.notification-actions-button.destructive:hover>.notification-actions-button-highlight{background-color:rgba(245,86,86,.1)}.notification-actions-button.destructive:active>.notification-actions-button-highlight{background-color:rgba(245,86,86,.2)}.notification-actions-button-highlight{padding:4px 12px;border-radius:12px;display:-ms-inline-flexbox;display:inline-flex;transition:background-color 150ms cubic-bezier(0,0,.2,1)}.notification-progress-ring{top:0;left:0;right:0;bottom:0;width:100%;height:100%;position:absolute}.notification-progress-circle{-webkit-transform:rotate(-90deg);-ms-transform:rotate(-90deg);transform:rotate(-90deg);-webkit-transform-origin:50% 50%;-ms-transform-origin:50% 50%;transform-origin:50% 50%;stroke:#657385}"]
            }] }
];
/** @nocollapse */
BbNotificationsItem.ctorParameters = () => [
    { type: NgZone },
    { type: BbPlatform },
    { type: ChangeDetectorRef }
];
BbNotificationsItem.propDecorators = {
    notification: [{ type: Input }],
    dismissButtonText: [{ type: Input }],
    progressElementRef: [{ type: ViewChild, args: ['progress',] }],
    getClass: [{ type: HostBinding, args: ['class',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbNotificationsModule {
    /**
     * @param {?} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('BbNotificationsModule is already loaded. Import it in the AppModule only');
        }
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: BbNotificationsModule,
            providers: [
                { provide: BbNotificationsConfig, useValue: config }
            ]
        };
    }
}
BbNotificationsModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [BbNotificationsList, BbNotificationsItem],
                entryComponents: [BbNotificationsList]
            },] }
];
/** @nocollapse */
BbNotificationsModule.ctorParameters = () => [
    { type: BbNotificationsModule, decorators: [{ type: Optional }, { type: SkipSelf }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbNotifications {
    /**
     * @param {?} _injector
     * @param {?} _platform
     * @param {?} _applicationRef
     * @param {?} _config
     * @param {?} _componentFactoryResolver
     */
    constructor(_injector, _platform, _applicationRef, _config, _componentFactoryResolver) {
        this._injector = _injector;
        this._platform = _platform;
        this._applicationRef = _applicationRef;
        this._config = _config;
        this._componentFactoryResolver = _componentFactoryResolver;
        // The data containing the notifications.
        this._notifications$ = new BehaviorSubject([]);
        this.getProperty = (/**
         * @param {?} property
         * @param {?} defaultValue
         * @return {?}
         */
        (property, defaultValue) => {
            return typeof property === 'undefined'
                ? defaultValue
                : property;
        });
        // Get the config.
        /** @type {?} */
        const config = this._config || {};
        // Set the config.
        this._defaultMode = this.getProperty(config.mode, 'prepend');
        this._defaultTimeout = this.getProperty(config.timeout, 8000);
        this._defaultDismiss = this.getProperty(config.dismiss, true);
        this._defaultDismissText = this.getProperty(config.dismissText, 'Dismiss');
        this._defaultPosition = this.getProperty(config.position, BbNotificationsPosition.TopRight);
        // Create the element that holds all notifications.
        this.createElement();
    }
    /**
     * @param {?} content
     * @param {?=} actions
     * @param {?=} timeout
     * @return {?}
     */
    success(content, actions = [], timeout = this._defaultTimeout) {
        return this.create({ content, timeout, actions, type: BbNotificationType.Success });
    }
    /**
     * @param {?} content
     * @param {?=} actions
     * @param {?=} timeout
     * @return {?}
     */
    error(content, actions = [], timeout = this._defaultTimeout) {
        return this.create({ content, timeout, actions, type: BbNotificationType.Error });
    }
    /**
     * @param {?} content
     * @param {?=} actions
     * @param {?=} timeout
     * @return {?}
     */
    warn(content, actions = [], timeout = this._defaultTimeout) {
        return this.create({ content, timeout, actions, type: BbNotificationType.Warning });
    }
    /**
     * @param {?} content
     * @param {?=} actions
     * @param {?=} timeout
     * @return {?}
     */
    info(content, actions = [], timeout = this._defaultTimeout) {
        return this.create({ content, timeout, actions, type: BbNotificationType.Info });
    }
    /**
     * @param {?} notification
     * @return {?}
     */
    create(notification) {
        // Compose a new notification item.
        /** @type {?} */
        const item = this.compose(notification);
        // Add the destroy function to the notification.
        item.destroy = (/**
         * @return {?}
         */
        () => this.pull(item));
        // Add the notification to the list
        // and return the item.
        return this.push(item);
    }
    /**
     * @private
     * @param {?} notification
     * @return {?}
     */
    push(notification) {
        // Get the current list.
        /** @type {?} */
        const oldList = this._notifications$.getValue();
        // Check which mode is activated.
        /** @type {?} */
        let newList = [...oldList];
        switch (this._defaultMode) {
            case 'append':
                newList = [...oldList, notification];
                break;
            case 'prepend':
            default:
                newList = [notification, ...oldList];
        }
        // Push the new notifications.
        this._notifications$.next(newList);
        // Return the notification for further use.
        return notification;
    }
    /**
     * @private
     * @param {?} notification
     * @return {?}
     */
    pull(notification) {
        // Get the current list.
        /** @type {?} */
        const newList = this._notifications$
            .getValue()
            .filter((/**
         * @param {?} item
         * @return {?}
         */
        item => item.id !== notification.id));
        // Push a new list.
        this._notifications$.next(newList);
    }
    /**
     * @private
     * @param {?} notification
     * @return {?}
     */
    compose(notification) {
        // Attach a random id to the notification.
        notification.id = Math.random().toString(36).substr(2, 10);
        // Set all properties.
        notification.type = this.getProperty(notification.type, BbNotificationType.Custom);
        notification.content = this.getProperty(notification.content, null);
        notification.timeout = this.getProperty(notification.timeout, this._defaultTimeout);
        notification.dismiss = this.getProperty(notification.dismiss, this._defaultDismiss);
        notification.dismissText = this.getProperty(notification.dismissText, this._defaultDismissText);
        // Return the composed notification.
        return notification;
    }
    /**
     * @private
     * @return {?}
     */
    createElement() {
        // Create the notification list and
        // get a reference to the component.
        this._ref = this._componentFactoryResolver
            .resolveComponentFactory(BbNotificationsList)
            .create(this._injector);
        // Save a reference to the notifications on
        // the component.
        this._ref.instance.notifications$ = this._notifications$.pipe(share());
        this._ref.instance.dismissText = this._defaultDismissText;
        this._ref.instance.position = this._defaultPosition;
        // Attach the component's view to the application
        // so that the change detection will run properly.
        this._applicationRef.attachView(this._ref.hostView);
        // If the platform is not a browser return.
        if (!this._platform.isBrowser) {
            return;
        }
        try {
            // Append the element to the DOM.
            document.body.appendChild(this._ref.location.nativeElement);
        }
        catch (_a) {
            // Don't do anything, because it must've failed.
        }
    }
}
BbNotifications.decorators = [
    { type: Injectable, args: [{
                providedIn: BbNotificationsModule
            },] }
];
/** @nocollapse */
BbNotifications.ctorParameters = () => [
    { type: Injector },
    { type: BbPlatform },
    { type: ApplicationRef },
    { type: BbNotificationsConfig, decorators: [{ type: Optional }] },
    { type: ComponentFactoryResolver }
];
/** @nocollapse */ BbNotifications.ngInjectableDef = defineInjectable({ factory: function BbNotifications_Factory() { return new BbNotifications(inject(INJECTOR), inject(BbPlatform), inject(ApplicationRef), inject(BbNotificationsConfig, 8), inject(ComponentFactoryResolver)); }, token: BbNotifications, providedIn: BbNotificationsModule });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbLocalizeConfig {
    constructor() {
        this.production = false;
        this.dictionary = [];
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbLocalize {
    /**
     * @param {?} _storage
     * @param {?} _language
     * @param {?} _interpolator
     * @param {?} _config
     */
    constructor(_storage, _language, _interpolator, _config) {
        this._storage = _storage;
        this._language = _language;
        this._interpolator = _interpolator;
        this._config = _config;
        this._currentDictionary = null;
        this._storageToken = 'bb-language-dictionary-name';
        this.log = (/**
         * @param {?} message
         * @return {?}
         */
        (message) => {
            console && console.warn && console.warn(message);
        });
        this.reload = (/**
         * @return {?}
         */
        () => {
            window && window.location && window.location.reload && window.location.reload();
        });
        // Check if at least one language was submitted to the service.
        if (this._config.dictionary.length < 1) {
            throw new Error('Localize: one or more languages should be supplied.');
        }
        // Save the dictionary.
        this._dictionary = this._config.dictionary;
        // Get the storage language.
        /** @type {?} */
        const storageToken = this._storage.get(this._storageToken);
        /** @type {?} */
        const storageLanguage = this._dictionary
            .find((/**
         * @param {?} item
         * @return {?}
         */
        item => item.name === storageToken));
        // Retrieve the supported language.
        /** @type {?} */
        const supportedLanguage = this._language.all.map((/**
         * @param {?} __0
         * @return {?}
         */
        ({ code, region }) => {
            /** @type {?} */
            const languageCode = [code, region].filter((/**
             * @param {?} data
             * @return {?}
             */
            data => !!data)).join('-');
            return this._dictionary.find((/**
             * @param {?} item
             * @return {?}
             */
            item => item.languages.includes(languageCode)));
        })).filter((/**
         * @param {?} item
         * @return {?}
         */
        item => !!item)).find((/**
         * @param {?} _
         * @param {?} index
         * @return {?}
         */
        (_, index) => index === 0));
        // Make sure the user always has a language by picking
        // the first language if the users browser does not
        // support any language.
        this._currentDictionary = storageLanguage || supportedLanguage || this._dictionary[0];
    }
    /**
     * @return {?}
     */
    getLocale() {
        // Check if the locale is not there, then just return the
        // default locale.
        if (!this._currentDictionary || !this._currentDictionary.locale) {
            return 'en-US';
        }
        // Return the locale that is set in the dictionary.
        return this._currentDictionary.locale;
    }
    /**
     * @param {?} language
     * @return {?}
     */
    setLanguage(language) {
        // Try to fetch the dictionary.
        /** @type {?} */
        const dictionary = this._dictionary.find((/**
         * @param {?} item
         * @return {?}
         */
        item => item.languages.includes(language)));
        // Check if the dictionary was found.
        if (!dictionary) {
            return this.log(`Localize: The language with name: "${language}" was not found.`);
        }
        // Save the language in the storage.
        this._storage.set(this._storageToken, dictionary.name);
        // We need to reload the application for the translations.
        this.reload();
    }
    /**
     * @return {?}
     */
    unsetLanguage() {
        // Remove the language in the local storage.
        this._storage.remove(this._storageToken);
        // We need to reload the application for the translations.
        this.reload();
    }
    /**
     * @param {?} token
     * @param {?=} data
     * @return {?}
     */
    translate(token, data = {}) {
        // Retrieve the translation from the dictionary.
        /** @type {?} */
        const translation = this.searchForTranslation(token);
        // Warn the user if the token was not
        // found in the dictionary.
        if (!translation && !this._config.production) {
            this.log(`Localize: The token with name: "${token}" was not found.`);
            return token;
        }
        // If the user did not define any extra data
        // just return the translation.
        if (Object.keys(data).length < 1) {
            return translation;
        }
        // The user has added extra data that needs to be
        // interpolated.
        return this._interpolator.interpolate(translation, data);
    }
    /**
     * @private
     * @param {?} token
     * @return {?}
     */
    searchForTranslation(token) {
        // Grab the dictionary data.
        /** @type {?} */
        const data = this._currentDictionary.data;
        // Check if the token is present in the dictionary.
        if (data[token]) {
            return data[token];
        }
        // Else search the dictionary for the translation
        // by splitting on the dot.
        /** @type {?} */
        const translation = token.split('.').filter((/**
         * @param {?} value
         * @return {?}
         */
        value => !!value)).reduce((/**
         * @param {?} dictionary
         * @param {?} value
         * @return {?}
         */
        (dictionary, value) => {
            if (dictionary === undefined || typeof dictionary === 'string') {
                return undefined;
            }
            return dictionary[value];
        }), data);
        // Check if the translation is a string
        // if this is true it can be returned.
        if (typeof translation === 'string') {
            return translation;
        }
        // This means the translation was not
        // correct.
        return null;
    }
}
BbLocalize.decorators = [
    { type: Injectable }
];
/** @nocollapse */
BbLocalize.ctorParameters = () => [
    { type: BbStorage },
    { type: BbLanguage },
    { type: BbInterpolator },
    { type: BbLocalizeConfig, decorators: [{ type: Optional }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbLocalizePipe {
    /**
     * @param {?} _localize
     */
    constructor(_localize) {
        this._localize = _localize;
    }
    /**
     * @param {?} value
     * @param {?=} args
     * @return {?}
     */
    transform(value, args = {}) {
        // Validate that the data exists.
        if (!this._localize || !value) {
            return value;
        }
        // Check if the args were formatted properly.
        if (typeof args !== 'object') {
            throw new Error('Localize: The pipe arguments were not formatted properly.');
        }
        // Try to translate the value.
        return this._localize.translate(value, args);
    }
}
BbLocalizePipe.decorators = [
    { type: Pipe, args: [{
                name: 'bbLocalize'
            },] }
];
/** @nocollapse */
BbLocalizePipe.ctorParameters = () => [
    { type: BbLocalize, decorators: [{ type: Optional }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BbLocalizeModule {
    /**
     * @param {?} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('BbLocalizeModule is already loaded. Import it in the AppModule only');
        }
    }
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: BbLocalizeModule,
            providers: [
                { provide: BbLocalizeConfig, useValue: config },
                { provide: LOCALE_ID, deps: [BbLocalize], useFactory: getLocale }
            ]
        };
    }
}
BbLocalizeModule.decorators = [
    { type: NgModule, args: [{
                providers: [BbLocalize],
                declarations: [BbLocalizePipe],
                exports: [BbLocalizePipe]
            },] }
];
/** @nocollapse */
BbLocalizeModule.ctorParameters = () => [
    { type: BbLocalizeConfig, decorators: [{ type: Optional }, { type: SkipSelf }] }
];
/**
 * @param {?} localize
 * @return {?}
 */
function getLocale(localize) {
    // This is done in a exported function to prevent the
    // "lambda is not supported" error.
    return localize.getLocale();
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { BbPlatform, BbLanguage, BbStorage, BbImage, BbInterpolator, BbDashboard, BbDashboardHeader, BbDashboardSidebar, BbDashboardSidebarGroup, BbDashboardSidebarItem, BbDashboardModule, BbNotificationType, BbNotificationsConfig, BbNotificationsPosition, BbNotifications, BbNotificationsModule, BbLocalizeConfig, BbLocalize, BbLocalizePipe, getLocale, BbLocalizeModule, BbNotificationsItem as ɵc, BbNotificationsList as ɵa, notificationAnimation as ɵb };

//# sourceMappingURL=bravobit-bb-foundation.js.map